@@ -15,11 +15,11 @@ BEGIN {
     # this is an attempt to get around it...
 
     # for the cpan shell, and other parsers
-    $VERSION = 'v0.16';
+    $VERSION = 'v0.17';
 
     # for actual inspection
     ${VERSION} 
-        = qv('0.16'); 
+        = qv('0.17'); 
 };
 
 # Ensure we get detailed errors while starting up.
@@ -244,7 +244,7 @@ UR::Object::Type->define(
         composite_id_separator           => { is => 'Text', len => 2 , default_value => "\t", is_optional => 1},        
         
         # details used by the managment of the "real" entity outside of the app (persistence) 
-        table_name                       => { is => 'Text', len => 256, is_optional => 1, source => 'data dictionary' },        
+        table_name                       => { is => 'Text', len => undef, is_optional => 1, source => 'data dictionary' },        
         query_hint                       => { is => 'Text', len => 1024 , is_optional => 1},        
         id_sequence_generator_name       => { is => 'Text', len => 256, is_optional => 1, source => 'data dictionary', doc => 'override the default choice for sequence generator name' },
 
@@ -547,7 +547,7 @@ UR - rich declarative transactional objects
 
 =head1 VERSION
 
-This document describes UR version v0.16.
+This document describes UR version v0.17.
 
 =head1 SYNOPSIS
 
@@ -578,12 +578,12 @@ This document describes UR version v0.16.
         ],
         data_source => 'MyDB1',
         table_name => 'Animal'
-    }
+    };
 
     class Cat { 
         is => 'Animal', 
         has => [
-            feet    => { is => 'Number' },
+            feet    => { is => 'Number', default_value => 4 },
             fur     => { is => 'Text', valid_values => [qw/fluffy scruffy/] },
         ],
         data_source => 'MyDB1',
@@ -672,9 +672,10 @@ Next, define a data source representing your database, Music/DataSource/DB1.pm
     use Music;
     
     class Music::DataSource::DB1 {
-        is => ['UR::DataSource::Mysql'],
+        is => ['UR::DataSource::MySQL', 'UR::Singleton'],
         has_constant => [
-            server  => { value => 'mysql.example.com' },
+            server  => { value => 'database=music' },
+            owner   => { value => 'music' },
             login   => { value => 'mysqluser' },
             auth    => { value => 'mysqlpasswd' },
         ]
@@ -683,7 +684,7 @@ Next, define a data source representing your database, Music/DataSource/DB1.pm
     or to get something going quickly, SQLite has smart defaults...
 
     class Music::DataSource::DB1 {
-        is => 'UR::DataSource::SQLite',
+        is => ['UR::DataSource::SQLite', 'UR::Singleton'],
     };
     
 
@@ -699,7 +700,7 @@ Create a class to represent artists, who have many CDs, in Music/Artist.pm
             cds  => { is => 'Music::Cd', is_many => 1, reverse_as => 'artist' }
         ],
         data_source => 'Music::DataSource::DB1',
-        table_name => 'ARTISTS',
+        table_name => 'ARTIST',
     };
 
 Create a class to represent CDs, in Music/Cd.pm
@@ -716,7 +717,7 @@ Create a class to represent CDs, in Music/Cd.pm
             artist_name => { via => 'artist', to => 'name' },
         ],
         data_source => 'Music::DataSource::DB1',
-        table_name => 'CDS',
+        table_name => 'CD',
     };
 
 
@@ -750,15 +751,14 @@ Regardless, if the classes and database tables are present, you can then use the
     my @cds_2007 = Music::Cd->get(year => 2007, artist => $first_artist);
    
     # Use non-equality operators:
-    my @same_some_artists = Music::Artist->get(
-        'name like' => 'John%',
+    my @some_cds = Music::Cd->get(
         'year between' => ['2004','2009']
     );
 
     # This will use a JOIN with the ARTISTS table internally to filter
     # the data in the database.  @some_cds will contain Music::Cd objects.
     # As a side effect, related Artist objects will be loaded into the cache
-    my @some_cds = Music::Cd->get(
+    @some_cds = Music::Cd->get(
         year => '2007', 
         'artist_name like' => 'Bob%' 
     );
@@ -826,42 +826,44 @@ laboratory automation and analysis systems for high-throughput genomics.
  
  Scott Smith	    sakoht@cpan.org	    Orginal Architecture
  Anthony Brummett   brummett@cpan.org	Primary Development
-
- Craig Pohl
- Todd Hepler
+ Josh McMichael
+ Eric Clark                              
+ Nathan Nutter                          
  Ben Oberkfell
- Kevin Crouse
+ Eddie Belter
+ Feiyu Du
  Adam Dukes
+ Brian Derickson    
+ Craig Pohl
+ Gabe Sanderson
+ Todd Hepler
+ Jason Walker
+ James Weible
  Indraniel Das
  Shin Leong
- Eddie Belter
  Ken Swanson
  Scott Abbott
  Alice Diec
  William Schroeder
- Eric Clark
  Shawn Leonard
  Lynn Carmichael
- Jason Walker
  Amy Hawkins
- Gabe Sanderson
- James Weible
- James Eldred
  Michael Kiwala
+ Kevin Crouse
  Mark Johnson
  Kyung Kim
  Jon Schindler
  Justin Lolofie
- Chris Harris
  Jerome Peirick
  Ryan Richt
  John Osborne
- Josh McMichael
+ Chris Harris
+ James Eldred
  David Dooling
  
 =head1 LICENCE AND COPYRIGHT
 
-Copyright (C) 2002-2009 Washington University in St. Louis, MO.
+Copyright (C) 2002-2010 Washington University in St. Louis, MO.
 
 This sofware is licensed under the same terms as Perl itself.
 See the LICENSE file in this distribution.
