@@ -21,17 +21,21 @@ sub evaluate_subject_and_values {
         @property_values = @{$property_values[0]};
     }
 
-    #my $property_value = $subject->$property_name;
-    no warnings;
-    #for (@$comparison_value) {
-    #    return 1 if ($property_value eq $_ ? 1 : '');
-    #}
+    no warnings qw(numeric uninitialized);
     foreach my $comparison_value (@$comparison_value) {
+        my $cv_is_number = Scalar::Util::looks_like_number($comparison_value);
+
         foreach my $property_value ( @property_values ) {
-            return 1 if ($property_value eq $comparison_value);
+            my $pv_is_number = Scalar::Util::looks_like_number($property_value);
+
+            if ($cv_is_number and $pv_is_number) {
+                return 1 if ($property_value == $comparison_value);
+            } else {
+                return 1 if ($property_value eq $comparison_value);
+            }
         }
     }
-    return;
+    return '';
 }
 
 
