@@ -7,7 +7,6 @@ use warnings;
 use IO::File;
 
 require UR;
-our $VERSION = "0.44_01"; # UR $VERSION;
 
 UR::Object::Type->define(
     class_name => __PACKAGE__,
@@ -19,34 +18,6 @@ UR::Object::Type->define(
     is_transactional => 1,
 );
 
-sub changed_object {
-    my $self = shift;
-    my $changed_obj;
-    my $changed_aspect = $self->changed_aspect;
-    if ($changed_aspect eq "delete" or $changed_aspect eq "unload") {
-        my $undo_data = $self->undo_data;
-        unless (defined $undo_data) {
-            $undo_data = '';
-        }
-        $changed_obj = eval "no strict; no warnings; " . $undo_data;
-        my $error = $@;
-        bless($changed_obj, 'UR::DeletedRef') if (ref $changed_obj); # changed class so that UR::Object::DESTROY is not called on a "fake" UR::Object
-        if ($error) {
-            Carp::confess("Error reconstructing $changed_aspect data for @_: $error");
-        }
-    }
-    else {
-        $changed_obj = $self->changed_class_name->get($self->changed_id);
-    }
-
-    if (defined $changed_obj) {
-        return $changed_obj;
-    }
-    else {
-        return;
-    }
-}
-
 sub undo {
     my $self = shift;
     my $changed_class_name = $self->changed_class_name;
@@ -65,7 +36,7 @@ sub undo {
     # Allow reversal of those methods to indirectly reverse ghost changes.
     if ($changed_class_name =~ /::Ghost/) {
         if ($changed_aspect !~ /^(create|delete)(_object|)$/) {
-            Carp::confess("Unlogged change on ghost? $self");
+            Carp::confess("Unlogged change on ghost? @_");
         }
         return 1;
     }
@@ -80,20 +51,32 @@ sub undo {
         }
     }
 
-    my $changed_obj = $self->changed_object();
-    return unless $changed_obj;
+    my $changed_obj;
+    if ($changed_aspect eq "delete" or $changed_aspect eq "unload") {
+        $undo_data = '' unless defined $undo_data;
+        $changed_obj = eval "no strict; no warnings; " . $undo_data;
+        if ($@) {
+            Carp::confess("Error reconstructing $changed_aspect data for @_: $@");
+        }
+        return unless $changed_obj;
+    }
+    else {
+        $changed_obj = $changed_class_name->get($changed_id);
+    }
     # TODO: if no changed object, die?
 
 
-    if ($changed_aspect eq "__define__") {
-        $changed_obj->unload();
+    if ($changed_aspect eq "_create_object") {
+        #$changed_obj->_delete_object();
+    }
+    elsif ($changed_aspect eq "_delete_object") {
+        #$changed_obj = $changed_class_name->_create_object(%$changed_obj);
+    }
+    elsif ($changed_aspect eq "__define__") {
+        UR::Object::unload($changed_obj);
     }
     elsif ($changed_aspect eq "create") {
-        if ($changed_obj->isa('UR::Observer')) {
-            UR::Observer::delete($changed_obj);  # Observers have state that needs to be cleaned up
-        } else {
-            UR::Object::delete($changed_obj);
-        }
+        UR::Object::delete($changed_obj);
     }
     elsif ($changed_aspect eq "delete") {
         my %stored;
@@ -108,7 +91,6 @@ sub undo {
         for my $key (keys %stored) {
             $changed_obj->{$key} = $stored{$key};
         }
-        $changed_obj->{'_change_count'}--;  # it was incremented when delete() was called on the object
     }
     elsif ($changed_aspect eq "load") {
         UR::Object::unload($changed_obj);
@@ -116,16 +98,16 @@ sub undo {
     elsif ($changed_aspect eq "load_external") {
     }
     elsif ($changed_aspect eq "unload") {
-        $changed_obj = $UR::Context::current->_construct_object($changed_class_name,%$changed_obj);
+        $changed_obj = UR::Object::_create_object($changed_class_name,%$changed_obj);
         UR::Object::__signal_change__($changed_obj,"load") if $changed_obj;
     } elsif ($changed_aspect eq "commit") {
         if ($changed_obj->isa('UR::Context::Transaction')) {
             UR::Object::unload($changed_obj);
         } else {
-            Carp::confess(q(Cannot undo 'commit' on a non-software transaction));
+            Carp::confess();
         }
     } elsif ($changed_aspect eq "rollback") {
-        Carp::confess(q(Cannot undo 'rollback'));
+        Carp::confess();
     } elsif ($changed_aspect eq 'rewrite_module_header') {
         my $VAR1;
         eval $undo_data;
@@ -150,24 +132,10 @@ sub undo {
         # regular property
         if ($changed_obj->can($changed_aspect)) {
             $changed_obj->$changed_aspect($undo_data);
-            $changed_obj->{'_change_count'} -= 2;  # 2 because the line above will actually increment the counter, too
         }
     }
 
-    $changed_obj->{'_change_count'} = 0 if ($changed_obj->{'_change_count'} and $changed_obj->{'_change_count'} < 0);
-
     return 1;
 }
 
-sub __rollback__ {
-    my $self = shift;
-    my $changed_aspect = $self->changed_aspect;
-    if($changed_aspect eq 'external_change') {
-        $self->undo;
-        $self->delete;
-    } else {
-        return $self->SUPER::__rollback__;
-    }
-}
-
 1;
