@@ -1,18 +1,19 @@
+
 package UR::Namespace::Command::Define::Class;
+
 use strict;
 use warnings;
 use UR;
-our $VERSION = "0.44_01"; # UR $VERSION;
 
 class UR::Namespace::Command::Define::Class {
-    is => 'UR::Namespace::Command::Base',
+    is => 'UR::Namespace::Command',
     has => [
-        names => {
+        extends => { doc => "The base class.  Defaults to UR::Object.", default_value => 'UR::Object' },
+        bare_args => {
             is_optional => 1,
             is_many => 1,
             shell_args_position => 1
-        },
-        extends => { doc => "The base class.  Defaults to UR::Object.", default_value => 'UR::Object' },
+        }
     ],
     doc => 'Add one or more classes to the current namespace'
 };
@@ -39,12 +40,14 @@ EOS
 
 sub execute {
     my $self = shift;
-    my @class_names = $self->names;
+    my @class_names = $self->bare_args;
     unless (@class_names) {
         $self->error_message("No class name(s) provided!");
         return;
     }
 
+    $self->_init or return;
+
     my $namespace = $self->namespace_name;
     my $is = $self->extends || 'UR::Object';
     my $parent_class_meta = UR::Object::Type->get($is);
