@@ -30,7 +30,7 @@ sub xsl_template_files {
        -e $root_path . $pf ? $pf : (-e $root_path . $df ? $df : undef)
     } $self->all_subject_classes_ancestry;
 
-    my @found_xsl_names = grep { 
+    my @found_xsl_names = grep {
         defined
     } @xsl_names;
 
@@ -42,20 +42,21 @@ sub _generate_content {
 
     my $subject = $self->subject();
     return '' unless $subject;
-    
+
     my $xml_doc = XML::LibXML->createDocument();
     $self->_xml_doc($xml_doc);
 
     # the header line is the class followed by the id
     my $object = $xml_doc->createElement('object');
     $xml_doc->setDocumentElement($object);
-    
+
     $object->addChild( $xml_doc->createAttribute('type', $self->subject_class_name) );
-    $object->addChild( $xml_doc->createAttribute('id', $subject->id) );
-    
+
+    $object->addChild( $xml_doc->createAttribute('id', $subject->id ) );
+
     my $display_name = $object->addChild( $xml_doc->createElement('display_name') );
     $display_name->addChild( $xml_doc->createTextNode($subject->__display_name__) );
-    
+
     my $label_name = $object->addChild( $xml_doc->createElement('label_name' ));
     $label_name->addChild( $xml_doc->createTextNode($subject->__label_name__) );
 
@@ -64,47 +65,54 @@ sub _generate_content {
         my $isa = $types->addChild( $xml_doc->createElement('isa') );
         $isa->addChild( $xml_doc->createAttribute('type', $c) );
     }
-    
+
     unless ($self->_subject_is_used_in_an_encompassing_view()) {
         # the content for any given aspect is handled separately
         my @aspects = $self->aspects;
         if (@aspects) {
             for my $aspect (sort { $a->number <=> $b->number } @aspects) {
                 next if $aspect->name eq 'id';
-                
+
                 my $aspect_node = $self->_generate_content_for_aspect($aspect);
+
                 $object->addChild( $aspect_node ) if $aspect_node; #If aspect has no values, it won't be included
             }
         }
     }
 
-
 #From the XML::LibXML documentation:
 #If $format is 1, libxml2 will add ignorable white spaces, so the nodes content is easier to read. Existing text nodes will not be altered
 #If $format is 2 (or higher), libxml2 will act as $format == 1 but it add a leading and a trailing line break to each text node.
-    return $xml_doc->toString(1);
+
+    my $doc_string = $xml_doc->toString(1);
+
+    # remove invalid XML entities
+    $doc_string =~ s/[^\x09\x0A\x0D\x20-\x{D7FF}\x{E000}-\x{FFFD}\x{10000}-\x{10FFFF}]//go;
+
+    return $doc_string;
+
 }
 
 sub _generate_content_for_aspect {
     # This does two odd things:
     # 1. It gets the value(s) for an aspect, then expects to just print them
-    #    unless there is a delegate view.  In which case, it replaces them 
+    #    unless there is a delegate view.  In which case, it replaces them
     #    with the delegate's content.
     # 2. In cases where more than one value is returned, it recycles the same
     #    view and keeps the content.
-    # 
+    #
     # These shortcuts make it hard to abstract out logic from toolkit-specifics
 
     my $self = shift;
     my $aspect = shift;
 
     my $subject = $self->subject;
-    my $xml_doc = $self->_xml_doc;  
+    my $xml_doc = $self->_xml_doc;
     my $aspect_name = $aspect->name;
-    
+
     my $aspect_node = $xml_doc->createElement('aspect');
     $aspect_node->addChild( $xml_doc->createAttribute('name', $aspect_name) );
-    
+
     my $aspect_meta = $self->subject_class_name->__meta__->property($aspect_name);
 
     my @value;
@@ -113,7 +121,7 @@ sub _generate_content_for_aspect {
     };
     if ($@) {
         my ($file,$line) = ($@ =~ /at (.*?) line (\d+)$/m);
-        
+
         my $exception = $aspect_node->addChild( $xml_doc->createElement('exception') );
         $exception->addChild( $xml_doc->createAttribute('file', $file) );
         $exception->addChild( $xml_doc->createAttribute('line', $line) );
@@ -121,11 +129,11 @@ sub _generate_content_for_aspect {
 
         return $aspect_node;
     }
-    
+
     if (@value == 0) {
-        return; 
+        return;
     }
-        
+
     if (Scalar::Util::blessed($value[0])) {
         unless ($aspect->delegate_view) {
             eval {
@@ -136,7 +144,7 @@ sub _generate_content_for_aspect {
             }
         }
     }
-    
+
     # Delegate to a subordinate view if needed.
     # This means we replace the value(s) with their
     # subordinate widget content.
@@ -144,7 +152,7 @@ sub _generate_content_for_aspect {
         foreach my $value ( @value ) {
             $delegate_view->subject($value);
             $delegate_view->_update_view_from_subject();
-            
+
             if ($delegate_view->can('_xml_doc') and $delegate_view->_xml_doc) {
                 my $delegate_xml_doc = $delegate_view->_xml_doc;
                 my $delegate_root = $delegate_xml_doc->documentElement;
@@ -160,7 +168,7 @@ sub _generate_content_for_aspect {
                 my $delegate_xml_doc = $parser->parse_string($aspect_text);
                 $aspect_node = $delegate_xml_doc->documentElement;
                 $xml_doc->adoptNode( $aspect_node );
-            }            
+            }
         }
     }
     else {
@@ -169,7 +177,7 @@ sub _generate_content_for_aspect {
             if (ref($value)) {
                 my $d = XML::Dumper->new;
                 my $xmlrep = $d->pl2xml($value);
-                
+
                 my $parser = XML::LibXML->new;
                 my $ref_xml_doc = $parser->parse_string($xmlrep);
                 my $ref_root = $ref_xml_doc->documentElement;
@@ -177,11 +185,11 @@ sub _generate_content_for_aspect {
                 $aspect_node->addChild( $ref_root );
             } else {
                 my $value_node = $aspect_node->addChild( $xml_doc->createElement('value') );
-                
+
                 unless(defined $value) {
                     $value = '';
                 }
-                
+
                 $value_node->addChild( $xml_doc->createTextNode($value) );
             }
         }
@@ -207,7 +215,7 @@ sub _resolve_default_aspects {
 
 =head1 NAME
 
-UR::Object::View::Default::Xml - represent object state in XML format 
+UR::Object::View::Default::Xml - represent object state in XML format
 
 =head1 SYNOPSIS
 
@@ -219,7 +227,7 @@ UR::Object::View::Default::Xml - represent object state in XML format
         'id',
         'name',
         'qty_on_hand',
-        'outstanding_orders' => [   
+        'outstanding_orders' => [
           'id',
           'status',
           'customer' => [
@@ -233,7 +241,7 @@ UR::Object::View::Default::Xml - represent object state in XML format
   $xml1 = $v->content;
 
   $o->qty_on_hand(200);
-  
+
   $xml2 = $v->content;
 
 =head1 DESCRIPTION
